/**
	(module summary)

	License: $(LICENSE)
	Copyright: Copyright ©2007 Sönke Ludwig, All rights reserved.
	Authors: Sönke Ludwig
*/
module sfc.intf.win32.objidl;

version(Windows){
	public import sfc.intf.win32.windows;
	public import std.c.windows.com;

	alias const(IID)* REFIID;

	alias IUnknown LPUNKNOWN;
	alias IMoniker LPMONIKER;
	alias IStream LPSTREAM;
	alias IPersist LPPERSIST;
	alias IPersistStream LPPERSISTSTREAM;
	alias IBindCtx LPBINDCTX, LPBC;
	alias IEnumMoniker LPENUMMONIKER;

	struct  STATSTG {
		LPOLESTR pwcsName;
		DWORD type;
		ULARGE_INTEGER cbSize;
		FILETIME mtime;
		FILETIME ctime;
		FILETIME atime;
		DWORD grfMode;
		DWORD grfLocksSupported;
		CLSID clsid;
		DWORD grfStateBits;
		DWORD reserved;
	}

	struct BIND_OPTS {
		DWORD cbStruct;
		DWORD grfFlags;
		DWORD grfMode;
		DWORD dwTickCountDeadline;
	}
	alias BIND_OPTS* LPBIND_OPTS;

	extern (C) {
		extern IID IID_ISequentialStream;
		extern IID IID_IStream;
		extern IID IID_IMarshal;
		extern IID IID_IStdMarshalInfo;
		extern IID IID_IMalloc;
		extern IID IID_IMallocSpy;
		extern IID IID_IMessageFilter;
		extern IID IID_IPersist;
		extern IID IID_IPersistStream;
		extern IID IID_IRunningObjectTable;
		extern IID IID_IBindCtx;
		extern IID IID_IMoniker;
		extern IID IID_IPersistStorage;
		extern IID IID_IPersistFile;
		extern IID IID_IAdviseSink;
		extern IID IID_IAdviseSink2;
		extern IID IID_IDataObject;
		extern IID IID_IDataAdviseHolder;
		extern IID IID_IStorage;
		extern IID IID_IRootStorage;
		extern IID IID_IRpcChannelBuffer;
		extern IID IID_IRpcProxyBuffer;
		extern IID IID_IRpcStubBuffer;
		extern IID IID_IPSFactoryBuffer;
		extern IID IID_ILockBytes;
		extern IID IID_IExternalConnection;
		extern IID IID_IRunnableObject;
		extern IID IID_IROTData;
		extern IID IID_IChannelHook;
		extern IID IID_IPropertyStorage;
		extern IID IID_IPropertySetStorage;
		extern IID IID_IClientSecurity;
		extern IID IID_IServerSecurity;
		extern IID IID_IClassActivator;
		extern IID IID_IFillLockBytes;
		extern IID IID_IProgressNotify;
		extern IID IID_ILayoutStorage;
		extern IID IID_IGlobalInterfaceTable;

		/*extern FMTID FMTID_SummaryInformation;
		extern FMTID FMTID_DocSummaryInformation;
		extern FMTID FMTID_UserDefinedProperties;*/
	}

	interface IEnumString : public IUnknown {
		  HRESULT Next(ULONG, LPOLESTR*, ULONG*);
		  HRESULT Skip(ULONG);
		  HRESULT Reset();
		  HRESULT Clone(IEnumString*);
	}

	interface IEnumMoniker : public IUnknown {
		  HRESULT Next(ULONG, IMoniker*, ULONG*);
		  HRESULT Skip(ULONG);
		  HRESULT Reset();
		  HRESULT Clone(IEnumMoniker*);
	}

	interface ISequentialStream : public IUnknown {
		HRESULT Read(void*, ULONG, ULONG*);
		HRESULT Write(void* , ULONG, ULONG*);
	}

	interface IStream : public ISequentialStream {
		HRESULT Seek(LARGE_INTEGER, DWORD, ULARGE_INTEGER*);
		HRESULT SetSize(ULARGE_INTEGER);
		HRESULT CopyTo(IStream, ULARGE_INTEGER, ULARGE_INTEGER*, ULARGE_INTEGER*);
		HRESULT Commit(DWORD);
		HRESULT Revert();
		HRESULT LockRegion(ULARGE_INTEGER, ULARGE_INTEGER, DWORD);
		HRESULT UnlockRegion(ULARGE_INTEGER, ULARGE_INTEGER, DWORD);
		HRESULT Stat(STATSTG*, DWORD);
		HRESULT Clone(LPSTREAM*);
	}

	interface IPersist : public IUnknown {
		HRESULT GetClassID(CLSID*);
	}

	interface IPersistStream : public IPersist {
		HRESULT IsDirty();
		HRESULT Load(IStream);
		HRESULT Save(IStream, BOOL);
		HRESULT GetSizeMax(PULARGE_INTEGER);
	}

	interface IRunningObjectTable : public IUnknown {
		HRESULT Register(DWORD, LPUNKNOWN, LPMONIKER, PDWORD);
		HRESULT Revoke(DWORD);
		HRESULT IsRunning(LPMONIKER);
		HRESULT GetObject(LPMONIKER, LPUNKNOWN*);
		HRESULT NoteChangeTime(DWORD, LPFILETIME);
		HRESULT GetTimeOfLastChange(LPMONIKER, LPFILETIME);
		HRESULT EnumRunning(IEnumMoniker*);
	}

	interface IBindCtx : public IUnknown {
		HRESULT RegisterObjectBound(LPUNKNOWN);
		HRESULT RevokeObjectBound(LPUNKNOWN);
		HRESULT ReleaseBoundObjects();
		HRESULT SetBindOptions(LPBIND_OPTS);
		HRESULT GetBindOptions(LPBIND_OPTS);
		HRESULT GetRunningObjectTable(IRunningObjectTable*);
		HRESULT RegisterObjectParam(LPOLESTR, IUnknown);
		HRESULT GetObjectParam(LPOLESTR, IUnknown*);
		HRESULT EnumObjectParam(IEnumString*);
		HRESULT RevokeObjectParam(LPOLESTR);
	}

	interface IMoniker: public IPersistStream {
		HRESULT BindToObject(IBindCtx, IMoniker, REFIID, PVOID*);
		HRESULT BindToStorage(IBindCtx, IMoniker, REFIID, PVOID*);
		HRESULT Reduce(IBindCtx, DWORD, IMoniker*, IMoniker*);
		HRESULT ComposeWith(IMoniker, BOOL, IMoniker*);
		HRESULT Enum(BOOL, IEnumMoniker*);
		HRESULT IsEqual(IMoniker);
		HRESULT Hash(PDWORD);
		HRESULT IsRunning(IBindCtx, IMoniker, IMoniker);
		HRESULT GetTimeOfLastChange(IBindCtx, IMoniker, LPFILETIME);
		HRESULT Inverse(IMoniker*);
		HRESULT CommonPrefixWith(IMoniker, IMoniker*);
		HRESULT RelativePathTo(IMoniker, IMoniker*);
		HRESULT GetDisplayName(IBindCtx, IMoniker, LPOLESTR*);
		HRESULT ParseDisplayName(IBindCtx, IMoniker, LPOLESTR, ULONG*, IMoniker*);
		HRESULT IsSystemMoniker(PDWORD);
	}

}
