/**
	(module summary)

	License: $(LICENSE)
	Copyright: Copyright ©2007-2008 Sönke Ludwig, All rights reserved.
	Authors: Sönke Ludwig
*/
module sfc.intf.win32.windows;

version(Win32)
{
	public import std.c.windows.windows;

	extern(Windows):

	// winerror.h
	enum {
		WAIT_TIMEOUT = 258
	}

	// windef.h
	struct POINTS {
		SHORT x;
		SHORT y;
	}

	struct SIZE {
		LONG        cx;
		LONG        cy;
	}
	alias SIZE* PSIZE, LPSIZE;

	enum {
		SWP_NOSIZE         = 0x0001,
		SWP_NOMOVE         = 0x0002,
		SWP_NOZORDER       = 0x0004,
		SWP_NOREDRAW       = 0x0008,
		SWP_NOACTIVATE     = 0x0010,
		SWP_FRAMECHANGED   = 0x0020,  /* The frame changed: send WM_NCCALCSIZE */
		SWP_SHOWWINDOW     = 0x0040,
		SWP_HIDEWINDOW     = 0x0080,
		SWP_NOCOPYBITS     = 0x0100,
		SWP_NOOWNERZORDER  = 0x0200,  /* Don't do owner Z ordering */
		SWP_NOSENDCHANGING = 0x0400,  /* Don't send WM_WINDOWPOSCHANGING */
	}

	// winbase
	alias ULONG SIZE_T;
	enum {
		GMEM_FIXED = 0x0000,
		GMEM_MOVEABLE = 0x0002,
		GMEM_ZEROINIT = 0x0040,
	}
	HGLOBAL GlobalAlloc(UINT uFlags, SIZE_T dwBytes);
	LPVOID GlobalLock(HGLOBAL hMem);
	BOOL GlobalUnlock(HGLOBAL hMem);

	HANDLE CreateEventA(void* lpEventAttributes, BOOL bManualReset, BOOL bInitialState, LPCSTR lpName);
	BOOL PulseEvent(HANDLE hEvent);



	// winnt
	union LARGE_INTEGER {
		struct {
			uint LowPart;
			int  HighPart;
		}
		long QuadPart;
	}
	alias LARGE_INTEGER* PLARGE_INTEGER;

	union ULARGE_INTEGER {
		struct {
			uint LowPart;
			uint HighPart;
		}
		ulong QuadPart;
	}
	alias ULARGE_INTEGER* PULARGE_INTEGER;

	// winuser
	enum {
		WM_MOUSEWHEEL = 0x020A,
		WM_MOUSELEAVE = 0x02A3,
		WM_USER = 0x0400,

		WHEEL_DELTA = 120
	}

	enum {
		TME_HOVER = 0x00000001,
		TME_LEAVE = 0x00000002,
		//#if(WINVER >= 0x0500)
		TME_NONCLIENT =  0x00000010,
		//#endif /* WINVER >= 0x0500 */
		TME_QUERY = 0x40000000,
		TME_CANCEL = 0x80000000
	}
	struct TRACKMOUSEEVENT {
		DWORD cbSize;
		DWORD dwFlags;
		HWND hwndTrack;
		DWORD dwHoverTime;
	}

	BOOL PostMessageA(HWND hwnd, UINT msg, WPARAM wPara, LPARAM lParam);
	BOOL WaitMessage();
	BOOL DestroyWindow(HWND hWnd);
	HWND GetDesktopWindow();
	BOOL SetWindowPos(HWND hWnd, HWND hWndInsertAfter, int X, int Y, int cx, int cy, UINT uFlags);
	BOOL SetWindowTextA(HWND hWnd, const(char)* lpString);
	HWND SetCapture( HWND hWnd );
	BOOL ReleaseCapture();
	BOOL TrackMouseEvent(TRACKMOUSEEVENT* lpEventTrack);

	enum {
		CF_TEXT = 1,
		CF_UNICODETEXT = 13,
	}
	BOOL OpenClipboard(HWND hWndNewOwner);
	BOOL CloseClipboard();
	HANDLE GetClipboardData(UINT uFormat);
	BOOL IsClipboardFormatAvailable(UINT format);
	HANDLE SetClipboardData(UINT uFormat, HANDLE hMem);
	BOOL EmptyClipboard();

	// gdi32
	POINTS MAKEPOINTS(LPARAM lp){ return *cast(POINTS*)&lp; }
	int ChoosePixelFormat(HDC hdc, const PIXELFORMATDESCRIPTOR *ppfd);
	BOOL SetPixelFormat(HDC hdc, int format, const PIXELFORMATDESCRIPTOR * ppfd);

	// mmsystem
	alias UINT MMRESULT;

	/* joystick error return values */
	enum {
		JOYERR_BASE         = 160,
		JOYERR_NOERROR      = (0),                  /* no error */
		JOYERR_PARMS        = (JOYERR_BASE+5),      /* bad parameters */
		JOYERR_NOCANDO      = (JOYERR_BASE+6),      /* request not completed */
		JOYERR_UNPLUGGED    = (JOYERR_BASE+7),      /* joystick is unplugged */
	}

	/* constants used with JOYINFO and JOYINFOEX structures and MM_JOY* messages */
	enum {
		JOY_BUTTON1       = 0x0001,
		JOY_BUTTON2       = 0x0002,
		JOY_BUTTON3       = 0x0004,
		JOY_BUTTON4       = 0x0008,
		JOY_BUTTON1CHG    = 0x0100,
		JOY_BUTTON2CHG    = 0x0200,
		JOY_BUTTON3CHG    = 0x0400,
		JOY_BUTTON4CHG    = 0x0800,
	}

	/* constants used with JOYINFOEX */
	enum {
		JOY_BUTTON5       = 0x00000010,
		JOY_BUTTON6       = 0x00000020,
		JOY_BUTTON7       = 0x00000040,
		JOY_BUTTON8       = 0x00000080,
		JOY_BUTTON9       = 0x00000100,
		JOY_BUTTON10      = 0x00000200,
		JOY_BUTTON11      = 0x00000400,
		JOY_BUTTON12      = 0x00000800,
		JOY_BUTTON13      = 0x00001000,
		JOY_BUTTON14      = 0x00002000,
		JOY_BUTTON15      = 0x00004000,
		JOY_BUTTON16      = 0x00008000,
		JOY_BUTTON17      = 0x00010000,
		JOY_BUTTON18      = 0x00020000,
		JOY_BUTTON19      = 0x00040000,
		JOY_BUTTON20      = 0x00080000,
		JOY_BUTTON21      = 0x00100000,
		JOY_BUTTON22      = 0x00200000,
		JOY_BUTTON23      = 0x00400000,
		JOY_BUTTON24      = 0x00800000,
		JOY_BUTTON25      = 0x01000000,
		JOY_BUTTON26      = 0x02000000,
		JOY_BUTTON27      = 0x04000000,
		JOY_BUTTON28      = 0x08000000,
		JOY_BUTTON29      = 0x10000000,
		JOY_BUTTON30      = 0x20000000,
		JOY_BUTTON31      = 0x40000000,
		JOY_BUTTON32      = 0x80000000,
	}

	/* constants used with JOYINFOEX structure */
	enum : WORD {
		JOY_POVCENTERED       = cast(WORD)-1,
		JOY_POVFORWARD        = 0,
		JOY_POVRIGHT          = 9000,
		JOY_POVBACKWARD       = 18000,
		JOY_POVLEFT           = 27000,
	}

	enum {
		JOY_RETURNX           = 0x00000001,
		JOY_RETURNY           = 0x00000002,
		JOY_RETURNZ           = 0x00000004,
		JOY_RETURNR           = 0x00000008,
		JOY_RETURNU           = 0x00000010,     /* axis 5 */
		JOY_RETURNV           = 0x00000020,     /* axis 6 */
		JOY_RETURNPOV         = 0x00000040,
		JOY_RETURNBUTTONS     = 0x00000080,
		JOY_RETURNRAWDATA     = 0x00000100,
		JOY_RETURNPOVCTS      = 0x00000200,
		JOY_RETURNCENTERED    = 0x00000400,
		JOY_USEDEADZONE       = 0x00000800,
		JOY_RETURNALL         = (JOY_RETURNX | JOY_RETURNY | JOY_RETURNZ |
                                 JOY_RETURNR | JOY_RETURNU | JOY_RETURNV |
                                 JOY_RETURNPOV | JOY_RETURNBUTTONS),
		JOY_CAL_READALWAYS    = 0x00010000,
		JOY_CAL_READXYONLY    = 0x00020000,
		JOY_CAL_READ3         = 0x00040000,
		JOY_CAL_READ4         = 0x00080000,
		JOY_CAL_READXONLY     = 0x00100000,
		JOY_CAL_READYONLY     = 0x00200000,
		JOY_CAL_READ5         = 0x00400000,
		JOY_CAL_READ6         = 0x00800000,
		JOY_CAL_READZONLY     = 0x01000000,
		JOY_CAL_READRONLY     = 0x02000000,
		JOY_CAL_READUONLY     = 0x04000000,
		JOY_CAL_READVONLY     = 0x08000000,
	}

	/* joystick ID constants */
	enum {
		JOYSTICKID1       = 0,
		JOYSTICKID2       = 1,
	}

	/* joystick driver capabilites */
	enum {
		JOYCAPS_HASZ          = 0x0001,
		JOYCAPS_HASR          = 0x0002,
		JOYCAPS_HASU          = 0x0004,
		JOYCAPS_HASV          = 0x0008,
		JOYCAPS_HASPOV        = 0x0010,
		JOYCAPS_POV4DIR       = 0x0020,
		JOYCAPS_POVCTS        = 0x0040,
	}

	struct joyinfoex_tag {
		DWORD dwSize;                /* size of structure */
		DWORD dwFlags;               /* flags to indicate what to return */
		DWORD dwXpos;                /* x position */
		DWORD dwYpos;                /* y position */
		DWORD dwZpos;                /* z position */
		DWORD dwRpos;                /* rudder/4th axis position */
		DWORD dwUpos;                /* 5th axis position */
		DWORD dwVpos;                /* 6th axis position */
		DWORD dwButtons;             /* button states */
		DWORD dwButtonNumber;        /* current button number pressed */
		DWORD dwPOV;                 /* point of view state */
		DWORD dwReserved1;           /* reserved for communication between winmm & driver */
		DWORD dwReserved2;           /* reserved for future expansion */
	}
	alias joyinfoex_tag JOYINFOEX;

	UINT joyGetNumDevs();
	MMRESULT joyGetPosEx( UINT uJoyID, JOYINFOEX* pji);
	MMRESULT joyReleaseCapture( UINT uJoyID);
	MMRESULT joySetCapture( HWND hwnd, UINT uJoyID, UINT uPeriod, BOOL fChanged);
}
